package jat;

import java.util.ArrayList;
import java.util.List;

public class CountDown {

	private int value;
	private int startValue = 10;

	private boolean running = false;
	private boolean stop = false;

	private List<CountDownListener> listeners = new ArrayList<>();

	public int getStartValue() {
		return startValue;
	}

	public void setStartValue(int startValue) {
		this.startValue = startValue;
	}

	public int getValue() {
		return value;
	}

	private void setValue(int value) {
		this.value = value;
		fireValueChanged();
	}

	public void stop(){
		if(running){
			stop = true;
		}
	}
	
	public void start() {
		setValue(startValue);
		stop = false;
		if (!running) {
			running = true;
			Thread t = new Thread(() -> {
				while (value > 0 && !stop) {
					try {
						Thread.sleep(1000);
					} catch (InterruptedException e) {
						Thread.currentThread().interrupt();
					}
					setValue(getValue() - 1);
				}
				running = false;
				if(!stop){
					fireTimeOut();
				}
			});
			t.start();
		}
	}

	public void addCountDownListener(CountDownListener listener) {
		listeners.add(listener);
	}

	public void removeCountDownListener(CountDownListener listener) {
		listeners.remove(listener);
	}

	public void fireValueChanged() {
		CountDownEvent event = new CountDownEvent(this, getValue());
		for (CountDownListener countDownListener : listeners) {
			countDownListener.valueChanged(event);
		}
	}

	public void fireTimeOut() {
		CountDownEvent event = new CountDownEvent(this, getValue());
		for (CountDownListener countDownListener : listeners) {
			countDownListener.timeOut(event);
		}
	}
}
